﻿using System;
using System.Web.Mvc;
using System.Web.Security;
using DotNetOpenAuth.ApplicationBlock;
using DotNetOpenAuth.OAuth;
using MvcGallery3.Utils;
using TwitterAuth.Framework;
using TwitterAuth.ViewModels.Auth;

namespace TwitterAuth.Controllers
{
    public class AuthController : OAuthControllerBase<AuthenticationOnlyTokenManager>
    {
        public AuthController()
        {
            TokenManager.SetConfigKeyAndSecret("TwitterConsumerKey", "TwitterConsumerSecret");
        }

        /// <summary>
        /// Displays the login view
        /// </summary>
        /// <returns>HTML view</returns>
        [HttpGet]
        public ActionResult LogOn()
        {
            return View();
        }

        /// <summary>
        /// Gets posted credentials and performs actual validation.
        /// </summary>
        /// <param name="model">Credentials and related information</param>
        /// <param name="returnUrl">Originally requested URL</param>
        /// <param name="defaultAction">Action to default when no return URL is specified</param>
        /// <param name="defaultController">Controller to default when no return URL is specified</param>
        /// <returns>Error view or requested redirect</returns>
        [HttpPost]
        public ActionResult LogOn(LogOnViewModel model, String returnUrl, String defaultAction="Index", String defaultController="Home")
        {
            var isValidReturnUrl = IsValidReturnUrl(returnUrl);

            if (!ModelState.IsValid)
            {
                ModelState.AddModelError("", "The user name or password provided is incorrect.");
                return View(model);
            }

            // Validate and proceed
            if (Membership.ValidateUser(model.UserName, model.Password))
            {
                FormsAuthentication.SetAuthCookie(model.UserName, model.RememberMe);
                if (isValidReturnUrl)
                {
                    return Redirect(returnUrl);
                }
                else
                {
                    return RedirectToAction(defaultAction, defaultController);
                }
            }

            // If we got this far, something failed, redisplay form
            return View(model);
        }

        /// <summary>
        /// Logs off and redirects to the specified view.
        /// </summary>
        /// <param name="defaultAction">Action to redirect</param>
        /// <param name="defaultController">Controller to redirect</param>
        /// <returns>Redirect</returns>
        public ActionResult LogOff(String defaultAction="Index", String defaultController="Home")
        {
            FormsAuthentication.SignOut();
            return RedirectToAction(defaultAction, defaultController);
        }

        /// <summary>
        /// Logs off and redirects to the specified route.
        /// </summary>
        /// <param name="defaultRoute">Route to redirect</param>
        /// <returns>Redirect</returns>
        public ActionResult LogOffToRoute(String defaultRoute)
        {
            FormsAuthentication.SignOut();
            return RedirectToRoute(defaultRoute); 
        }

        /// <summary>
        /// Relies on Twitter to authenticate users.
        /// </summary>
        /// <returns>Redirect</returns>
        public ActionResult TwitterLogOn()
        {
            var twitterClient = new WebConsumer(TwitterConsumer.ServiceDescription, TokenManager);

            // Create the callback URL for Twitter
            var callbackUri = Request.ReplacePath("Auth/TwitterAuthenticated").AsUri();

            // Place the request
            twitterClient.Channel.Send(twitterClient.PrepareRequestUserAuthorization(callbackUri, null, null));

            // You should be redirected to Twitter at this point ...
            return new EmptyResult();  
        }

        /// <summary>
        /// Process the response from Twitter
        /// </summary>
        /// <returns>Logged screen</returns>
        public ActionResult TwitterAuthenticated()
        {
            // Process the response
            var twitterClient = new WebConsumer(TwitterConsumer.ServiceDescription, TokenManager);
            var accessTokenResponse = twitterClient.ProcessUserAuthorization();

            // Is Twitter calling back with authorization?
            if (accessTokenResponse != null)
            {
                // Extract the access token and username for use throughout the site
                var cookie = accessTokenResponse.CreateAuthCookie();
                Response.Cookies.Add(cookie);
            }
            else
            {
                // If the request doesn't come with an authentication token redirect to the login page
                return RedirectToAction("LogOn", "Auth");
            }

            // If the authentication is successful, redirect to the home page
            return RedirectToAction("Index", "Home");
        }

        

        #region Helpers
        /// <summary>
        /// Verify that the provided return URL is valid
        /// </summary>
        /// <param name="returnUrl">URL to check</param>
        /// <returns>Boolean</returns>
        private Boolean IsValidReturnUrl(String returnUrl)
        {
            return Url.IsLocalUrl(returnUrl) && 
                returnUrl.Length > 1 && 
                returnUrl.StartsWith("/") && 
                !returnUrl.StartsWith("//") && 
                !returnUrl.StartsWith("/\\");
        }
        #endregion
    }
}
