using System;
using System.Collections.Generic;
using System.Configuration;
using DotNetOpenAuth.OAuth.ChannelElements;
using DotNetOpenAuth.OAuth.Messages;

namespace TwitterAuth.Framework
{
    public abstract class ConfigTokenManagerBase : IConsumerTokenManager
    {
        // Config entries to read consumer key and secret for the application
        protected String ConsumerKeyConfigEntry;
        protected String ConsumerSecretConfigEntry;

        // This is your backing store (and you should be changed to something persistent like a DB)
        protected readonly Dictionary<String, String> TokensAndSecrets = new Dictionary<String, String>();

        /// <summary>
        /// Saves configuration entries for application's consumer key/secret.
        /// </summary>
        /// <param name="consumerKeyConfigEntry">Config entry for consumer key</param>
        /// <param name="consumerSecretConfigEntry">Config entry for consumer secret</param>
        public void SetConfigKeyAndSecret(String consumerKeyConfigEntry = "ConsumerKey", String consumerSecretConfigEntry = "ConsumerSecret")
        {
            ConsumerKeyConfigEntry = consumerKeyConfigEntry;
            ConsumerSecretConfigEntry = consumerSecretConfigEntry;
        }
        public void SetKeyConfigEntry(String consumerKeyConfigEntry = "ConsumerKey")
        {
            ConsumerKeyConfigEntry = consumerKeyConfigEntry;
        }
        public void SetSecretConfigEntry(String consumerSecretConfigEntry = "ConsumerSecret")
        {
            ConsumerSecretConfigEntry = consumerSecretConfigEntry;
        }

        /// <summary>
        /// Retrieves the consumer key 
        /// </summary>
        public String ConsumerKey
        {
            get 
            {
                var key = ConfigurationManager.AppSettings[ConsumerKeyConfigEntry];
                if (String.IsNullOrEmpty(key))
                    throw new ArgumentNullException("ConsumerKey");
                return key;
            }
        }

        /// <summary>
        /// Your consumer secret--get from Twitter, read from config
        /// </summary>
        public String ConsumerSecret
        {
            get
            {
                var secret = ConfigurationManager.AppSettings[ConsumerSecretConfigEntry];
                if (String.IsNullOrEmpty(secret))
                    throw new ArgumentNullException("ConsumerSecret");
                return secret;
            }
        }

        /// <summary>
        /// Deletes a request token and its associated secret and stores a new access token and secret.
        /// </summary>
        /// <param name="consumerKey">The Consumer that is exchanging its request token for an access token.</param>
        /// <param name="requestToken">The Consumer's request token that should be deleted/expired.</param>
        /// <param name="accessToken">The new access token that is being issued to the Consumer.</param>
        /// <param name="accessTokenSecret">The secret associated with the newly issued access token.</param>
        public void ExpireRequestTokenAndStoreNewAccessToken(String consumerKey, String requestToken, String accessToken, String accessTokenSecret)
        {
            StoreAccessTokenCore(consumerKey, requestToken, accessToken, accessTokenSecret);
        }

        /// <summary>
        /// Gets the Token Secret given a request or access token.
        /// </summary>
        /// <param name="token">The request or access token.</param>
        /// <returns>
        /// The secret associated with the given token.
        /// </returns>
        public String GetTokenSecret(String token)
        {
            return GetTokenSecretCore(token);
        }

        /// <summary>
        /// Classifies a token as a request token or an access token.
        /// </summary>
        /// <param name="token">The token to classify.</param>
        /// <returns>Request or Access token, or invalid if the token is not recognized.</returns>
        public TokenType GetTokenType(String token)
        {
            return GetTokenTypeCore(token);
        }

        /// <summary>
        /// Stores a newly generated unauthorized request token, secret, and optional
        /// application-specific parameters for later recall.
        /// </summary>
        /// <param name="request">The request message that resulted in the generation of a new unauthorized request token.</param>
        /// <param name="response">The response message that includes the unauthorized request token.</param>
        /// <remarks>
        /// Request tokens stored by this method SHOULD NOT associate any user account with this token.
        /// It usually opens up security holes in your application to do so.  Instead, you associate a user
        /// account with access tokens (not request tokens) in the <see cref="ExpireRequestTokenAndStoreNewAccessToken"/>
        /// method.
        /// </remarks>
        public void StoreNewRequestToken(UnauthorizedTokenRequest request, ITokenSecretContainingMessage response)
        {
            StoreNewRequestTokenCore(request, response);
        }

        #region Overridables

        protected abstract void StoreAccessTokenCore(String consumerKey, String requestToken, String accessToken, String accessTokenSecret);
        protected abstract String GetTokenSecretCore(String token);
        protected virtual TokenType GetTokenTypeCore(String token)
        {
            throw new NotImplementedException();
        }
        protected abstract void StoreNewRequestTokenCore(UnauthorizedTokenRequest request, ITokenSecretContainingMessage response);
        #endregion
    }
}